//-----------------------------------------------
//
// top16_example_1.cpp  
// 
// Using the top16 DLL from Visual C++
// This is a console application tested on Visual C++ 2010 Express
// FTD2XX.dll and top16.dll must be present in the execution directory
//
//
//  Modifications:
//  13-October-2010 Created
//
// 
//

#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <stdint.h>
#include <string.h>
#include <time.h>

/*----------------- Functions and definitions for top16.dll --------------------*/

typedef void (WINAPI*cfunc)();


typedef _int32 (WINAPI *FT_ListDevices) (void*, void*, int) ;
typedef _int32 (WINAPI *ListTop16Boards) (char stringArray[][20]);
typedef _int32 (WINAPI *CloseBoard) (unsigned _int32 Handle)  ;
typedef _int32 (WINAPI *OpenBoard) (char* board) ;
typedef _int32 (WINAPI *SetOutputs) (unsigned _int32 Handle, unsigned _int8 mask, unsigned _int8 set) ;
typedef _int32 (WINAPI *SetOutputsB) (unsigned _int32 Handle, unsigned _int8 mask, unsigned _int8 set) ;
typedef _int32 (WINAPI *SetPWM)  (unsigned _int32 Handle, unsigned _int8 pwm, unsigned _int8 output) ;
typedef _int32 (WINAPI *GetInputs) (unsigned _int32 Handle) ;
typedef _int32 (WINAPI *readAnalogInput) (unsigned _int32 , _int16 input, unsigned _int8 gain) ;
typedef _int32 (WINAPI *dllversion)() ;

HINSTANCE top16DLL;

ListTop16Boards lpfnListTop16Boards ;
CloseBoard lpfnCloseBoard ;
OpenBoard lpfnOpenBoard ;
SetOutputs lpfnSetOutputs ;
GetInputs lpfnGetInputs ;
SetOutputsB lpfnSetOutputsB ;
SetPWM lpfnSetPWM ;
readAnalogInput lpfnreadAnalogInput ;
dllversion lpfndllversion ;

BOOL setupLibrary()
{      
    // Load the top16.DLL
    //
    top16DLL = LoadLibrary("top16.dll");
    if (top16DLL != NULL)
    {
      lpfnListTop16Boards = (ListTop16Boards)GetProcAddress(top16DLL, "ListTop16Boards");            
      lpfnCloseBoard = (CloseBoard) GetProcAddress(top16DLL, "CloseBoard") ;
      lpfnOpenBoard  = (OpenBoard) GetProcAddress(top16DLL, "OpenBoard") ;
      lpfnSetOutputs = (SetOutputs) GetProcAddress(top16DLL,"SetOutputs") ;
      lpfnSetOutputsB = (SetOutputsB) GetProcAddress(top16DLL,"SetOutputsB") ;
      lpfnGetInputs = (GetInputs) GetProcAddress(top16DLL, "GetInputs") ;
	  lpfnSetPWM = (SetPWM) GetProcAddress (top16DLL, "SetPWM");
	  lpfndllversion = (dllversion) GetProcAddress (top16DLL, "dllversion") ;
	  lpfnreadAnalogInput = (readAnalogInput) GetProcAddress (top16DLL, "readAnalogInput") ;
      
	  // were all the functions found in the dll ?
	  //
      if (!lpfnListTop16Boards || !lpfnCloseBoard || !lpfnOpenBoard || !lpfnSetOutputs || ! lpfnGetInputs || !lpfnSetOutputsB || !lpfndllversion || !lpfnreadAnalogInput)
      {
        // failed to load a function
        //
        FreeLibrary(top16DLL) ;
        return(FALSE) ;
        
      }
      return(TRUE) ;
       
    }
    return(FALSE) ;
}
       
/*------------------- End functions and definitions for top16.dll ------------------------*/     




// Main function for console application
//
int _tmain(int argc, _TCHAR* argv[])
{
	char boardNames[10][20] ; // expecting no more than 10 boards 


	if (setupLibrary() == FALSE)
	{
		printf("Failed to load top16.dll\n") ;
		return(-1) ;
	}

	// list the connected top16 boards
	//
	int found = (lpfnListTop16Boards)(boardNames) ;	

	for (int i = 0 ; i < found; i++)
	{
		if (boardNames[i] != NULL)
		{
			printf("Found: %s\n", boardNames[i]) ;
		}
	}

	// for each board found turn on output 1, 3, 5, 7
	//
	for (int i = 0 ; i < found; i++)
	{
		_int32 handle = lpfnOpenBoard(boardNames[i]) ; 
		if (handle > 0)
		{
			(lpfnSetOutputsB) (handle, 0xFF, 0x55) ; //binary 01010101
			lpfnCloseBoard(handle) ;
		}
		
	}
	// for each board read an analog input 1 at gain 1
	//
	for (int i = 0 ; i < found; i++)
	{
		_int32 handle = lpfnOpenBoard(boardNames[i]) ; 
		if (handle > 0)
		{
			// read the analog input 20 times
			//
			for(int  readCount = 0; readCount < 20; readCount++)
			{
				_int32 reading = (lpfnreadAnalogInput) (handle, 1, (unsigned _int8)'Z') ;
				printf("Analog Input1 reading = %ld \n", reading) ; 
			}
			
			lpfnCloseBoard(handle) ;
		}		
	}

	// release the DLL handle
	//
	FreeLibrary(top16DLL);


	// wait for Enter to be pressed
	//
	getchar() ;
	return 0;
}

